<?php
// task class for use with osticket tasklist mod
//
// created by leigh edwards
//
//
// TODO : Change task data from arrays to objects.
// TODO : change popup status and priority text color to match task view. 
// TODO : add function for assigning task to tickets or tickets to tasks
// TODO : add function to wrap tasks accross multiple days as aposed to spanning them
// TODO : finnish add and edit task forms with schedule options
// TODO : add functions to allow mutiple tasks in the same time slot if schedule viewed at department or admin level
// TODO : add functions to prevent overlap of tasks assigned to the same staff member
// TODO : allow mutiple staff members to be assigned to the same task and reflect this in the assigned to sections
// TODO : add options to assign as task leader and reflect in task views


class task {
	private $taskTable = "ost_tasks";
	private $statusTable = "ost_task_status";
	private $noteTable = "ost_task_notes";
	private $priorityList;
	private $formData;
	private $taskList;
	private $options;
	private $userObj;
	private $staffList;
	private $statusList;
	private $departmentList;
	private $task_id;
	private $scheduleObj;
	private $taskSchedule;
	private $timeSlotLength = 30; // length in mins
	private $imageMapCode;
	private $popupWidth = 400;
	private $usersImageFile;
	private $user_id;
	
	public $date;
	
	public $taskScheduleImageFile = "scheduleImage"; // path to file to store schedlue images in, must be writable
	public $isError = 0;
	public $msg = "";
	
	/**
	 * __construct function sets up all defaults
	 * abd pulls in needed data from the database
	 *
	 */
	function __construct() {
		$this->setDate ();
		$this->setPriorityList ();
		$this->setStaffList ();
		$this->setStatusList ();
		$this->setDepartmentList ();
	
	}
	public function getTaskId() {
		return $this->task_id;
	}
	
	public function setDate() {
		if ($_GET ['date']) {
			$this->date = date ( 'Y-m-d', strtotime ( $_GET ['date'] ) );
		} else {
			$this->date = date ( 'Y-m-d' );
		}
	}
	
	/**
	 * Returns the html code to display the ations menu
	 * on the edit task forms
	 *
	 * @return string
	 */
	public function getActionMenu() {
		return $this->actionMenu ();
	}
	
	/**
	 * returns the name of the given department
	 * Id
	 *
	 * @param int $dept_id
	 * @return string
	 */
	public function getDepartment($dept_id) {
		$out = "Department Fault";
		foreach ( $this->departmentList as $departmentArray ) {
			if ($departmentArray ['dept_id'] == $dept_id) {
				$out = $departmentArray ['dept_name'];
				return $out;
			}
		}
		return $out;
	}
	
	/**
	 * method to return the first and last name
	 * for the given staff Id
	 *
	 * @param int $id
	 * @return string
	 */
	public function getStaffName($id) {
		$name = "Not Assigned";
		foreach ( $this->staffList as $staff ) {
			if ($staff ['staff_id'] == $id) {
				$name = $staff ['firstname'] . " " . $staff ['lastname'];
				return $name;
			}
		}
		return $name;
	}
	
	/**
	 * method to return an array of tasks
	 * defined by $case. See $this->setTaskList
	 * for info on accepted values for $case
	 *
	 * @param string $case
	 * @return array
	 */
	public function getTaskList($case) {
		$this->setTaskList ( $case );
		return $this->taskList;
	}
	
	/**
	 * method to return html string to display a given task
	 * on the view task page
	 *
	 * @return string
	 */
	public function getTaskView() {
		
		If (! $task_id = $_GET ['task_id']) {
			$task_id = $_POST ['task_id'];
		}
		If ($this->taskexists ( $task_id )) {
			return $this->buildTaskView ( $task_id );
		} else {
			$this->setMsg ( 1, "Can not find selected task in the database" );
		}
	}
	
	/**
	 * method to return an array of staff names saved in $this->staffList
	 * by the setStaffList method
	 *
	 * @return array
	 */
	public function getStaffList() {
		return $this->staffList;
	}
	
	/**
	 * method to save a new task into the database
	 *
	 * @param array $formData
	 * @return true or false
	 */
	public function insertNewtask($formData) {
		// clean the data
		$this->formData = $this->validate_form_data ( $formData );
		//insert in to database
		return $this->addTaskToDatabase ();
	}
	
	public function insertNewNote($formData) {
		$this->formData = $formData;
		return $this->addNoteToDatabase ();
	}
	
	public function getUsersImageFile() {
		return $this->usersImageFile;
	}
	
	/**
	 * method to update edit task form data
	 * into the database
	 *
	 * @param string $formData
	 * @return true or false
	 */
	public function editTask($formData) {
		$this->formData = $this->validate_form_data ( $formData );
		return $this->updateTask ( $this->formData );
	}
	
	public function getStatusList() {
		return $this->statusList;
	}
	public function getStatusDropDownList($selection) {
		return $this->buildStatusDropDownList ( $selection );
	}
	
	public function getPriorityList() {
		return $this->priorityList;
	}
	
	public function getPriorityDropDownList($selection) {
		return $this->buildPriorityDropDownList ( $selection );
	}
	
	public function getDepartmentList() {
		return $this->departmentList;
	}
	
	public function setUser($userObj) {
		$this->userObj = $userObj;
	}
	
	public function getMsg() {
		return $this->msg;
	}
	/**
	 * method to perform given actions
	 * 
	 * 
	 */
	public function action($actions) {
		$this->setTaskId ( $actions ['task_id'] );
		switch ($actions ['do']) {
			case 'change_priority' :
				$this->setTaskPriority ( $this->task_id, $actions ['task_priority'] );
				break;
			case 'close' :
				$this->setMsg ( 0, "close" );
				break;
			case 'delete' :
				$this->deleteTask ( $this->task_id );
				break;
			default :
		}
	}
	
	/**
	 * method to return one task record
	 *
	 * @param int $task_id
	 * @return array
	 */
	public function getTaskRecord($task_id) {
		return $this->getTasks ( 'task_id', $task_id );
	}
	
	/**
	 * method to create xml and other schedule data
	 */
	public function buildSchedule() {
		require_once ('class.schedule.php');
		$this->scheduleObj = new Schedule ( );
		$xmlData = $this->buildScheduleXML ();
		if (! $this->scheduleObj->LoadXmlData ( $xmlData )) {
			echo "LoadXmlData Error<br>";
		}
		$fileName = "../images/" . $this->taskScheduleImageFile . $this->userObj->id . ".png";
		$this->usersImageFile = $fileName;
		$this->scheduleObj->Draw ( $this->usersImageFile );
	}
	
	public function getSlots($time) {
		return $this->timeToSlots ( $time );
	}
	
	/**
	 * method to return the xml for the schedule
	 * 
	 * @return string
	 */
	public function buildScheduleXML() {
		$xml = "<schedule blocks='00:00,00:30,01:00,01:30,02:00,02:30,03:00,03:30,04:00,04:30,05:00,05:30,06:00,06:30,07:00,";
		$xml .= "07:30,08:00,08:30,09:00,09:30,10:00,10:30,11:00,11:30,12:00,12:30,13:00,13:30,14:00,14:30,15:00,15:30,16:00,";
		$xml .= "16:30,17:00,17:30,18:00,18:30,19:00,19:30,20:00,20:30,21:00,21:30,22:00,22:30,23:00,23:30' ";
		$xml .= "days='";
		$days = "";
		$startDate = strtotime ( $this->getWeekStartDate ( $this->date ) );
		for($day = 0; $day <= 6; $day ++) {
			$date = mktime ( 0, 0, 0, date ( 'm', $startDate ), date ( 'd', $startDate ) + $day, date ( 'Y', $startDate ) );
			$days .= date ( 'D jS M y', $date );
			if ($day != 6) {
				$days .= ",";
			}
		}
		$xml .= $days . "'><settings>";
		$xml .= "<legend font='2' leftpadding='40' toppadding='20' rightpadding='0' bottompadding='0'/>";
		$xml .= "<block>\n";
		$xml .= "<title align='left' color='#000000' font='1' />\n";
		$xml .= "<description align='left' color='#000000' font='2' />\n";
		$xml .= "</block>\n";
		$xml .= "</settings>\n";
		
		// build task blocks
		$xml .= $this->getTaskBlocks ();
		$xml .= "</schedule>";
		return $xml;
	}
	/**
	 * method to return the code to display the image maps
	 *
	 * @return string
	 */
	public function geImageMap() {
		return $this->imageMapCode;
	}
	
	/**
	 * method to return a date in the week following the date passed
	 * 
	 * @param string $date
	 * @return string $date
	 */
	public function get_fdate($date) {
		list ( $year, $month, $day ) = explode ( '-', $this->date );
		$date = date ( 'Y-m-d', strtotime ( '+ 1 week', mktime ( 0, 0, 0, $month, $day, $year ) ) );
		return $date;
	}
	
	/**
	 * method to return a date in the week preceeding the date passed
	 * 
	 * @param string $date
	 * @return string $date
	 */
	public function get_bdate($date) {
		list ( $year, $month, $day ) = explode ( '-', $this->date );
		$date = date ( 'Y-m-d', strtotime ( '- 1 week', mktime ( 0, 0, 0, $month, $day, $year ) ) );
		return $date;
	}
	
	/**
	 * method to return the date for the sunday starting the week of the given date
	 * 
	 * @param string date as ('Y-m-d')
	 * @return string
	 */
	private function getWeekStartDate($date) {
		return date ( 'Y-m-d', mktime ( 0, 0, 0, date ( 'm', strtotime ( $date ) ), date ( 'd', strtotime ( $date ) ) - date ( 'w', strtotime ( $date ) ), date ( 'Y', strtotime ( $date ) ) ) );
	}
	
	/**
	 * method to return the date for the sunday starting the week of the given date
	 * 
	 * @param string date as ('Y-m-d')
	 * @return string
	 */
	private function getWeekEndDate($date) {
		return date ( 'Y-m-d', mktime ( 0, 0, 0, date ( 'm', strtotime ( $date ) ), date ( 'd', strtotime ( $date ) ) + (6 - date ( 'w', strtotime ( $date ) )), date ( 'Y', strtotime ( $date ) ) ) );
	}
	
	/**
	 * method to collect all task data and convert it to XML
	 * for use in buildScheduleXML()
	 *
	 * @return string
	 */
	private function getTaskBlocks() {
		// pull all tasks with time slots for given week and build xml code to return
		// start image maping
		// date for the week to be diaplayed, it can be any date within the desired week
		$date = $this->date;
		$startOfWeek = $this->getWeekStartDate ( $date );
		$endOfWeek = $this->getWeekEndDate ( $date );
		
		// TODO : change query to include all data for the required tasks from task table, task status and task schedule tables so we can reduce the overall number of queries
		// $query = "SELECT * FROM " . $this->taskTable . " WHERE start_day <= DATE_ADD(CAST('$date' AS DATE), INTERVAL(1-DAYOFWEEK(CAST('$date' AS DATE))) DAY)
		//	  AND DATE_ADD(start_day, INTERVAL day_span DAY) <= DATE_ADD(CAST('$date' AS DATE), INTERVAL(7-DAYOFWEEK(CAST('$date' AS DATE))) DAY)
		//  AND DATE_ADD(start_day, INTERVAL day_span DAY) >= DATE_ADD(CAST('$date' AS DATE), INTERVAL(7-DAYOFWEEK(CAST('$date' AS DATE))) DAY)
		//	  OR  DATE_ADD(start_day, INTERVAL day_span DAY) >= DATE_ADD(CAST('$date' AS DATE), 
		//  INTERVAL(1-DAYOFWEEK(CAST('$date' AS DATE))) DAY)";
		$query = "SELECT * FROM " . $this->taskTable . " WHERE start_day BETWEEN '$startOfWeek' AND '$endOfWeek'";
		$imageMapCode = "<map name=\"schedule\" id=\"schedule\">\n";
		$xml = "";
		$result = db_query ( $query );
		$resultArray = db_assoc_array ( $result );
		$overlapArray = $this->getOverlapTasks ( $startOfWeek, $endOfWeek );
		$i = 0;
		for($day = 1; $day <= 7; $day ++) {
			$xml .= "<day number='$day'>";
			foreach ( $resultArray as $result ) {
				If (date ( 'w', strtotime ( $this->revSqlDate ( $result ['start_day'] ) ) ) + 1 == $day) {
					$title = mb_substr ( $result ['task_title'], 0, 18 );
					if (strlen ( $title ) >= 18) {
						$title = $title . "...";
					}
					$at = $this->timeToSlots ( $result ['start_time'] );
					$blocks = $result ['time_slots'] - 1;
					$color = $this->getPrioritycolor ( $result ['task_priority'] );
					// set coordfor iamge map 
					$x = (($day - 1) * 110) + 40;
					$y = (($at - 1) * 18.5) + 20;
					$xx = $x + 110;
					$yy = $y + (20 * $result ['time_slots']);
					$xml .= "<block name='$title' at='$at' blockspan='$blocks' dayspan='0' dayoffset='0' blockoffset='0'  bgcolor='$color' alpha='70%'></block>";
					$imageMapCode .= "<area shape=\"rect\" onmouseout=\"hideTip()\" onMouseOver=\"doTooltip(event,$i)\" coords=\"$x,$y,$xx,$yy\" href=\"tasklist.php?t=view&task_id=" . $result ['task_id'] . "\" />\n";
					$scriptTwo .= $this->buildScriptTwo ( $result ['task_id'], $i );
					$i ++;
				}
			}
			$xml .= "</day>";
		}
		$imageMapCode .= "</map>";
		// inject roll over code
		require_once ('staff/rollOver.inc.php');
		$this->imageMapCode = $scriptOne . "\n" . $scriptTwo . "\n" . $scriptThree . "\n" . $imageMapCode;
		
		return $xml;
	
	}
	
	/**
	 * method to find tasks that span into and out of the week being displayed
	 */
	private function getOverlapTasks($startOfWeek, $endOfWeek) {
		// TODO: work out how to display tasks that overlap weeks and days correcly. May require creating more xml blocks to approximate wraparound
	}
	
	/**
	 * method to build roll over script
	 *
	 * @param int $task_id
	 * @param int $i
	 * @return string
	 */
	private function buildScriptTwo($task_id, $i) {
		$query = "SELECT * FROM " . $this->taskTable . " WHERE task_id = '$task_id'";
		//echo $query."<br>";
		$result = db_query ( $query );
		$resultArray = db_assoc_array ( $result );
		foreach ( $resultArray as $result ) {
			$ra = $result;
		}
		$color = $this->getPrioritycolor ( $ra ['task_priority'] );
		$dueDate = $this->revSqlDate ( $ra ['task_due_date'], "/" );
		$startDate = $this->revSqlDate ( $ra ['start_day'], "/" );
		$box = "<table width=\"100%\" class=\"ticketinfo\">";
		$box .= "<tr><td colspan=\"2\" align=\"center\"><strong>" . $ra ['task_title'] . "</strong>";
		$box .= "<tr><td>Priority: </td><td>" . $this->priority ( $ra ['task_priority'] ) . "</td></tr>";
		$box .= "<tr><td>Assigned to: </td><td>" . $this->getStaffName ( $ra ['task_owner'] ) . "</td></tr>";
		$box .= "<tr><td>Status: </td><td>" . $this->StatusCellData ( $ra ['task_status'], "default" ) . "</td></tr>";
		$box .= "<tr><td>Due Date: </td><td>" . $dueDate . "</td></tr>";
		$box .= "<tr><td>Start Date: </td><td>" . $startDate . "</td></tr>";
		$box .= "<tr><td>Information: </td><td>Task ID: " . $ra ['task_id'] . "</td></tr>";
		$box .= "<tr><td colspan=\"2\">" . $ra ['task_text'] . "</td></tr>";
		$box .= "</table>";
		return "messages[$i] = new Array('','$box','$color');";
	}
	
	/**
	 * method to convert a tasks start time to 
	 * a time slot inr
	 *
	 * @param string $start_time
	 * @return int
	 */
	private function timeToSlots($start_time) {
		$timeArray = explode ( ":", $start_time );
		$hour = $timeArray [0];
		$min = $timeArray [1];
		$slot_number = $hour * 2;
		If ($min == $this->timeSlotLength) {
			$slot_number ++;
		}
		$slot_number ++;
		return $slot_number;
	}
	
	/**
	 * method to do query to update a task in the database
	 *
	 * @param array $formData
	 * @return true or false
	 */
	private function updateTask($formData) {
		
		$query = "UPDATE " . $this->taskTable . " SET task_title='" . $formData ['task_title'] . "', task_owner='" . $formData ['task_owner'] . "', ";
		$query .= "task_text='" . $formData ['task_text'] . "', task_status='" . $formData ['task_status'] . "' , task_department='" . $formData ['department'] . "', ";
		$query .= "start_day = '" . $formData ['start_day'] . "', task_due_date='" . $formData ['task_due_date'] . "', task_priority='" . $formData ['task_priority'] . "' WHERE task_id='" . $formData ['task_id'] . "'";
		//echo $query;
		if ($result = db_query ( $query )) {
			$this->setMsg ( 0, "Task was Updated" );
			return true;
		} else {
			$this->setMsg ( 1, "Faild to update task record" );
			return false;
		}
	}
	
	/**
	 * method to set a tasks priority value in the database
	 *
	 * @param int $id
	 * @param int $priority
	 * @return true or false
	 */
	private function setTaskPriority($id, $priority) {
		$query = "UPDATE " . $this->taskTable . " SET task_priority='" . $priority . "' WHERE task_id = '" . $id . "'";
		if ($result = db_query ( $query )) {
			$this->setMsg ( 0, "Task Priority Set" );
			return true;
		} else {
			$this->setMsg ( 1, "Faild To set Task Priority" );
			return false;
		}
	}
	
	/**
	 * method to remove a task from the database
	 *
	 * @param unknown_type $task_id
	 * @return unknown
	 */
	private function deleteTask($task_id) {
		if (! $this->taskexists ( $task_id )) {
			// delete task		
			$this->setMsg ( 1, "Unable to delete seleced task" );
			return false;
		} else {
			$query = "DELETE FROM " . $this->taskTable . " WHERE task_id = '" . $task_id . "'";
			$result = db_query ( $query );
			$this->setMsg ( 0, "task deleted" );
			return true;
		}
	
	}
	// TODO : write setTaskOverDue method
	private function setTaskOverDue() {
	}
	// TODO : Write SetTaskStatus method
	private function setTaskStatus() {
	}
	
	/**
	 * method to set object task_id
	 *
	 * @param int $task_id
	 */
	private function setTaskId($task_id) {
		$this->task_id = $task_id;
	}
	
	/**
	 * method to set error message and type
	 * takes int of value 1 or 0 and message string
	 * 
	 * @param int $type 
	 * @param sting $msg
	 */
	private function setMsg($type, $msg) {
		$this->isError = $type;
		$this->msg .= $msg;
	}
	
	/**
	 * method to retun a html string to display a given task
	 *
	 * @param int $task_id
	 * @return string
	 */
	private function buildTaskView($task_id) {
		return $this->taskView ( $this->getTasks ( 'task_id', $task_id ) );
	}
	
	/**
	 * method to build html string of task view
	 *
	 * @param array $taskArray
	 * @return string
	 */
	private function taskView($taskArray) {
		
		foreach ( $taskArray as $task ) {
			$this->setTaskId ( $task ['task_id'] );
			$out = "<div id=\"ticketthread\">";
			$out .= "<table class='ticketinfo' align='center' border='0' cellpadding='3' cellspacing='1' width='100%'><tbody>";
			$out .= "<tr><td><div class=\"msg\">View Task</div></td><td></td><td class=\"msg\"><a href=\"tasklist.php?t=edit&task_id=" . $task ['task_id'] . "\" title=\"Edit Ticket\" class=\"Icon editTicket\">Edit Task</a></td><td></td></tr>";
			$out .= "<tr><th>Task :</th><td>" . $task ['task_title'] . "</td><th>Priority:</th>";
			if ($task ['task_priority'] >= 3) {
				$out .= "<td><BLINK><font color='" . $this->getPrioritycolor ( $task ['task_priority'] ) . "'>" . $this->priority ( $task ['task_priority'] ) . "</font></BLINK></td><tr>";
			} else {
				$out .= "<td>" . $this->priority ( $task ['task_priority'] ) . "</td><tr>";
			}
			$out .= "<tr><th>Assigned to:</th><td>" . $this->getStaffName ( $task ['task_owner'] ) . "</td><th>Due Date:</th><td>" . $this->revSqlDate ( $task ['task_due_date'] ) . "</td></tr>";
			$out .= "<tr><th>Department:</th><td>" . $this->getDepartment ( $task ['task_department'] ) . "</td><th>Status:</th>" . $this->StatusCellData ( $task ['task_status'] ) . "</tr>";
			$out .= "</td><td></td></tr><tbody></table>";
			
			$out .= $this->actionMenu ( $task ['task_id'] );
			
			$out .= "<table class='response' align='center' border='0' cellpadding='1' cellspacing='0' width='100%'>";
			$out .= "<tr><th>Task Details: </th></tr>";
			$out .= "<tr><td>" . $task ['task_text'] . "</td></tr></table>\n";
			$out .= $this->notes ( $task ['task_id'] );
			$out .= "</div>";
			// TODO : use msg to display error if task is over due
			return $out;
		
		}
	}
	
	/**
	 * method to return a html string for displaying all notes related to the 
	 * submitted taskid
	 * 
	 * @param int $task_id
	 * @return string
	 */
	private function notes($task_id) {
		$notesArray = $this->getNotes ( $task_id );
		$out = "";
		foreach ( $notesArray as $note ) {
			If (strlen ( $note ['task_text'] ) < 1) {
				$text = "no note text found";
			} else {
				$text = $note ['task_text'];
			}
			$date = date ( 'D, M j Y h:ia ', strtotime ( $note ['note_date'] ) );
			$out .= "<table class='message' align='center' border='0' cellpadding='1' cellspacing='0' width='100%'>";
			$out .= "<tr><th >$date - " . $this->getStaffName ( $note ['note_creator'] ) . "</th><th width=\"50\">";
			$out .= "<form action=\"tasklist.php\" method=\"post\">
            <input name=\"id\" value=\"" . $this->userObj->id . "\" type=\"hidden\">
            <input name=\"note_id\" value=\"" . $note ['note_id'] . "\" type=\"hidden\">
            <input name=\"t\" value=\"deletenote\" type=\"hidden\">
            <input name=\"task_id\" value=\"$task_id\" type=\"hidden\">
            <input class=\"button\" value=\"Delete\" onclick='return confirm(\"Are you sure you want to DELETE selected note?\");' type=\"submit\">
            </form>	";
			$out .= "</th></tr>";
			$out .= "<tr class=\"header\" ><td colspan='2'>" . $note ['note_title'] . "</td></tr>";
			$out .= "<tr ><td colspan='2'>$text</td></tr></table><br>";
		}
		return $out;
	}
	
	/**
	 * method to return an array of all notes for the 
	 * passed task id
	 * 
	 * @param int $task_id
	 * @return array
	 */
	private function getNotes($task_id) {
		$query = "SELECT * FROM " . $this->noteTable . " WHERE task_id = '$task_id'";
		$result = db_query ( $query );
		if (db_num_rows ( $result ) < 1) {
			return " ";
		} else {
			$resultArray = db_assoc_array ( $result ); // returns array af arrays
			return $resultArray;
		}
	}
	public function deleteNote($note_id) {
		if (! $this->noteExists ( $note_id )) {
			$this->setMsg ( 1, "Unable to delete seleced note" );
			return false;
		} else {
			$query = "DELETE FROM " . $this->noteTable . " WHERE note_id = '" . $note_id . "'";
			$result = db_query ( $query );
			$this->setMsg ( 0, "note deleted" );
			return true;
		}
	}
	
	/**
	 * method to check if given note id exists in the database
	 *
	 * @param int $task_id
	 * @return true or false
	 */
	private function noteExists($note_id) {
		$query = "SELECT task_id FROM " . $this->noteTable . " WHERE note_id =" . $note_id;
		$result = db_query ( $query );
		if (db_num_rows ( $result ) < 1) {
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * method to check if given task id exists in the database
	 *
	 * @param int $task_id
	 * @return true or false
	 */
	private function taskexists($task_id) {
		$query = "SELECT task_status FROM " . $this->taskTable . " WHERE task_id =" . $task_id;
		$result = db_query ( $query );
		if (db_num_rows ( $result ) < 1) {
			return false;
		} else {
			return true;
		}
	}
	/**
	 * method to set $options value in this object
	 *
	 * @param unknown_type $options
	 */
	private function setOptions($options) {
		$this->options = $options;
	}
	
	/**
	 * method to save a html string to display the task list
	 * in $this->taskList
	 * Takes a string used to pick which tasks to display
	 *
	 * @param string $case
	 */
	private function setTaskList($case) {
		// admin lists all tasks
		switch ($case) {
			case 'my' :
				$list = $this->getTasks ( 'task_owner', $this->userObj->id );
				break;
			case 'all' :
				$list = $this->getTasks ( 'all', '0' );
				break;
			case 'department' :
			default :
				$list = $this->getTasks ( 'department', $this->userObj->dept_id );
		}
		$this->taskList = $this->formatTaskList ( $list );
	}
	
	/**
	 * method to build a html string of a given array of tasks
	 *
	 * @param array $list
	 * @return string
	 */
	private function formatTaskList($list) {
		$out = "<table width='100%' border='0' cellspacing=0 cellpadding=2 class='dtable' align='center'>";
		$out .= "<tbody align='left' bgcolor='cdcdcd'><tr><th>Task</th><th>Assigned to</th><th>Task Status</th>";
		if ($this->userObj->isAdmin ()) {
			$out .= "<th>Department</th>";
		}
		$out .= "<th>Due Date</th><th>Priority</th></tr></tbody>";
		foreach ( $list as $task ) {
			$out .= "<tr bgcolor='" . $this->getPrioritycolor ( $task ['task_priority'] ) . "'>";
			$out .= "<td><a href=\"tasklist.php?t=view&task_id=" . $task ['task_id'] . "\">" . $task ['task_title'] . "</a></td>";
			$out .= "<td>" . $this->getStaffName ( $task ['task_owner'] ) . "</td>";
			$out .= $this->StatusCellData ( $task ['task_status'] );
			if ($this->userObj->isAdmin ()) {
				$out .= "<td>" . $this->getDepartment ( $task ['task_department'] ) . "</td>";
			}
			$out .= "<td>" . $this->revSqlDate ( $task ['task_due_date'] ) . "</td>";
			$out .= "<td>" . $this->priority ( $task ['task_priority'] ) . "</td>";
			$out .= "</tr>";
		}
		$out .= "</table>";
		return $out;
	}
	
	/**
	 * method to return an array of tasks
	 * Takes a sort case and value 
	 *
	 * @param string $list_by
	 * @param mixed $value
	 * @param string $order
	 * @return array or false on fail
	 */
	private function getTasks($list_by, $value, $order = "DESC") {
		// first build the basic query depentent on the value of $list_by
		switch ($list_by) {
			case 'department' :
				$query = "SELECT task_id, task_title, task_owner, task_text, task_status, task_department, task_due_date, task_priority FROM " . $this->taskTable;
				if (! $this->userObj->isAdmin ()) {
					$query .= " WHERE task_" . $list_by . " = '" . $value . "'";
				}
				break;
			case 'task_id' :
				$query = "SELECT * FROM " . $this->taskTable . " WHERE task_id ='" . $value . "'";
				break;
			case 'task_owner' :
				$query = "SELECT * FROM " . $this->taskTable . " WHERE task_owner ='" . $value . "'";
				break;
			case 'all' :
				$query = "SELECT task_id, task_title, task_owner, task_text, task_status, task_department, task_due_date, task_priority FROM " . $this->taskTable;
				break;
			default :
		
		}
		// add in the order statment
		$query .= " ORDER BY task_priority $order";
		// do the query
		if (($result = db_query ( $query )) && db_num_rows ( $result )) {
			if (db_num_rows ( $result ) == 0) {
				$this->setMsg ( 1, "No task found" );
				return FALSE;
			}
			$resultArray = db_assoc_array ( $result ); // returns array af arrays
			return $resultArray;
		} else {
			return FALSE;
		}
	
	}
	
	/**
	 * method to save an array of all staff names from the database
	 * in $this->staffList
	 * 
	 * @return true or false
	 */
	private function setStaffList() {
		$query = "SELECT staff_id, firstname, lastname FROM ost_staff WHERE isactive ='1' ORDER BY lastname DESC";
		if (($result = db_query ( $query )) && db_num_rows ( $result )) {
			$this->staffList = db_assoc_array ( $result );
			return true;
		} else {
			$this->staffList = FALSE;
			return false;
		}
	}
	
	/**
	 * method to save accoc array of all status data
	 * in $this->statusList
	 * 
	 * @return true or false
	 */
	private function setStatusList() {
		$query = "SELECT status_id, status, status_color FROM " . $this->statusTable;
		if (($result = db_query ( $query )) && db_num_rows ( $result )) {
			$this->statusList = db_assoc_array ( $result );
			return true;
		} else {
			$this->statusList = false;
			return false;
		}
	}
	
	private function StatusCellData($status_id, $view = "") {
		$out = "<td>Status Fault</td>";
		foreach ( $this->statusList as $statusArray ) {
			if ($statusArray ['status_id'] == $status_id) {
				if (! $view) {
					$out = "<td><font color='" . $statusArray ['status_color'] . "'>" . $statusArray ['status'] . "</font></td>";
				} else {
					$out = $statusArray ['status'];
				}
				return $out;
			}
		}
		return $out;
	}
	
	/**
	 * method to save an accoc array of departments in $this->departmentList
	 * else returns false
	 *
	 * @return true or false
	 */
	private function setDepartmentList() {
		$query = "SELECT dept_id, dept_name FROM " . DEPT_TABLE . " WHERE ispublic = 1";
		if (($result = db_query ( $query )) && db_num_rows ( $result )) {
			$this->departmentList = db_assoc_array ( $result );
			return TRUE;
		} else {
			$this->departmentList = FALSE;
			return FALSE;
		}
	
	}
	private function addNoteToDatabase() {
		$query = "INSERT INTO " . $this->noteTable . " (task_id, note_title, note_text, note_creator) VALUES ";
		$query .= "( '" . $this->formData ['task_id'] . "', '" . $this->formData ['note_title'] . "', '" . $this->formData ['note_text'] . "', '" . $this->formData ['note_creator'] . "' )";
		
		if ($result = db_query ( $query )) {
			$this->setMsg ( 0, "New task note was added to the system" );
			return TRUE;
		} else {
			$this->setMsg ( 1, "Faild to add new task note to the system" );
			return FALSE;
		}
	}
	
	/**
	 * method to submit a task into the database
	 * 
	 * Takes task data from $this->formData
	 *
	
	 */
	private function addTaskToDatabase() {
		// build the query
		$query = "INSERT INTO " . $this->taskTable . " (task_title, task_owner, task_text, task_status, task_creator, task_department, task_due_date, task_priority, time_slots , start_day, start_time, day_span) VALUES ";
		$query .= "( '" . $this->formData ['task_title'] . "', '" . $this->formData ['task_owner'] . "', '" . $this->formData ['task_text'] . "', '" . $this->formData ['task_status'] . "',";
		$query .= " '" . $this->formData ['task_creator'] . "', '" . $this->formData ['department'] . "', '" . $this->formData ['task_due_date'] . "', '" . $this->formData ['task_priority'] . "',";
		$query .= " '" . $this->formData ['time_slots'] . "', '" . $this->revDate ( $this->formData ['start_day'] ) . "', '" . $this->formData ['start_time'] . "', '" . $this->formData ['day_span'] . "' )";
		// do the query
		if ($result = db_query ( $query )) {
			$this->setMsg ( 0, "New task was added to the system" );
			return TRUE;
		} else {
			$this->setMsg ( 1, "Faild to add new task to the system" );
			return FALSE;
		}
	}
	
	/**
	 * method to validate all form data
	 * NOTE THIS METHOD IS NOT COMPLEATE AND MAY ALLOW INVALID DATE TO BE PASSED THROUGH
	 *
	 * @param array $formData
	 * @return array
	 */
	private function validate_form_data($formData) {
		// first pull form data from sent array and dump the rest
		// we do this just incase someone is trying to send extra 
		// form eliments and re-order them
		

		if ($formData ['task_id']) {
			$dataArray ['task_id'] = $formData ['task_id'];
		}
		// schedule date
		$dataArray ['task_title'] = $formData ['task_title'];
		$dataArray ['task_owner'] = $formData ['task_owner'];
		$dataArray ['task_text'] = $formData ['task_text'];
		$dataArray ['task_status'] = $formData ['task_status'];
		$dataArray ['task_creator'] = $formData ['task_creator'];
		$dataArray ['department'] = $formData ['department'];
		$dataArray ['task_due_date'] = $this->revDate ( $formData ['task_due_date'] );
		$dataArray ['task_priority'] = $formData ['task_priority'];
		$dataArray ['start_day'] = $this->revDate ( $formData ['start_day'] );
		$dataArray ['start_time'] = $formData ['start_time'];
		$dataArray ['time_slots'] = $formData ['time_slots'];
		// TODO : Make correction for time slots that extend past the end of day
		$dataArray ['day_span'] = $formData ['day_span'];
		//TODO: add in form validation
		

		return $dataArray;
	}
	private function correctTimeSlots($start_time, $time_slots) {
		// get time slot for start time
		$start_slot = $this->timeToSlots ( $start_time );
		$slotValue = $start_slot - 1 + $time_slots;
		If ($slotValue > 48) {
			$slots = 48 - $slotValue;
		} else {
			$slots = $time_slots;
		}
		return $time_slots;
	}
	/**
	 * method to return the task value for the 
	 * given priority_id
	 *
	 * @param int $priority_id
	 * @return string
	 */
	private function priority($priority_id) {
		$priorityList = $this->priorityList;
		foreach ( $priorityList as $priority ) {
			if ($priority ['priority_id'] == $priority_id) {
				return $priority ['priority_desc'];
			}
		}
		return "Unknown";
	}
	
	/**
	 * method to return the color code for the given priority_id
	 *
	 * @param int $priority_id
	 * @return int
	 */
	private function getPrioritycolor($priority_id) {
		//echo $priority_id;
		$priorityList = $this->priorityList;
		foreach ( $priorityList as $priority ) {
			if ($priority ['priority_id'] == $priority_id) {
				return $priority ['priority_color'];
			}
		}
		return "666666";
	}
	
	/**
	 * method to save an array of priorities in 
	 * $this->priorityList
	 *
	 */
	private function setPriorityList() {
		$this->priorityList = $this->buildPriorityList ();
	}
	
	/**
	 * method to return an accoc array of priorities
	 *
	 * @return array
	 */
	private function buildPriorityList() {
		$query = "SELECT * FROM ost_ticket_priority";
		if (($result = db_query ( $query )) && db_num_rows ( $result )) {
			$priorityList = db_assoc_array ( $result );
		}
		
		return $priorityList;
	}
	
/**
	 * method to return html string to display status list drop down
	 * Takes id of the default selection as $selection
	 * @param unknown_type $selection
	 * @return unknown
	 */
	private function buildStatusDropDownList($selection_id) {
		// build priority drop down
		$this->setStatusList ();
		$statusList = $this->getStatusList ();
		$optionList = "<select name=\"task_status\">";
		$op_id = 1;
		foreach ( $statusList as $status ) {
			$optionList .= "<OPTION value=\"" . $status ['status_id'] . "\"";
			If ($op_id == $selection_id) {
				$optionList .= " selected ";
			}
			$optionList .= ">" . $status ['status'] . "</OPTION>";
			$op_id ++;
		}
		$optionList .= "</select>";
		return $optionList;
	}
	
	
	/**
	 * method to return html string to display priority list drop down
	 * Takes id of the default selection as $selection
	 * @param unknown_type $selection
	 * @return unknown
	 */
	private function buildPriorityDropDownList($selection_id) {
		// build priority drop down
		$this->setPriorityList ();
		$priorityList = $this->getPriorityList ();
		$optionList = "<select name=\"task_priority\">";
		$op_id = 1;
		foreach ( $priorityList as $priority ) {
			$optionList .= "<OPTION value=\"" . $priority ['priority_id'] . "\"";
			If ($op_id == $selection_id) {
				$optionList .= " selected ";
			}
			$optionList .= ">" . $priority ['priority_desc'] . "</OPTION>";
			$op_id ++;
		}
		$optionList .= "</select>";
		return $optionList;
	}
	
	/**
	 * method to reverse the order of a given date
	 * and fix to mysql date format
	 * so DD/MM/YYYY becomes YYYY-MM-DD
	 *
	 * @param unknown_type $date
	 */
	private function revDate($date) {
		// first split the date string @ / into three parts
		$dateArray = explode ( '/', $date, 3 );
		// then reorder them to YYYY-MM-DD
		$revDate = array_reverse ( $dateArray );
		$i = 0;
		foreach ( $revDate as $eliment ) {
			$correctDate .= $eliment;
			if ($i < 2) {
				$correctDate .= "-";
			}
			$i ++;
		}
		return $correctDate;
	}
	
	/**
	 * method to revers dates taken from sql database
	 * so YYYY-MM-DD becomes DD-MM-YYYY
	 *
	 * @param unknown_type $date
	 */
	public function revSqlDate($date, $deliminator = "") {
		// first split the date string @ / into three parts
		$dateArray = explode ( '-', $date, 3 );
		// then reorder them to DD-MM-YYYY
		$revDate = array_reverse ( $dateArray );
		$i = 0;
		foreach ( $revDate as $eliment ) {
			$correctDate .= $eliment;
			if ($i < 2) {
				if (! $deliminator) {
					$correctDate .= "-";
				} else {
					$correctDate .= $deliminator;
				}
			}
			$i ++;
		}
		
		return $correctDate;
	}
	
	/**
	 * method to return html to display the action menu
	 *
	 * @return string
	 */
	private function actionMenu() {
		
		$task_id = $this->task_id;
		$out = "<form name='action' action='tasklist.php?" . $task_id . " method='post' class='inline'>";
		$out .= "<input name='t' value='action' type='hidden'>";
		$out .= "<input name='task_id' value='" . $task_id . "' type='hidden'>";
		$out .= "<span for='do'> &nbsp;<b>Action:</b></span>";
		$out .= "<select name='do' onchange=\"this.form.task_priority.disabled=strcmp(this.options[this.selectedIndex].value,'change_priority','reopen')?false:true;\">";
		$out .= "<option value=''>Select Action</option>";
		$out .= "<option value='change_priority'>Change Priority</option>";
		$out .= "<option value='delete'>Delete Task</option></select>";
		$out .= "<span for='task_priority'>Priority:</span>";
		$out .= "<select id='task_priority' name='task_priority' disabled='disabled'>";
		$out .= "<option value='0' selected='selected'>-Unchanged-</option>";
		$out .= "<option value='1'>Low</option>";
		$out .= "<option value='2'>Normal</option>";
		$out .= "<option value='3'>High</option>";
		$out .= "<option value='4'>Emergency</option></select>";
		$out .= "&nbsp;&nbsp;";
		$out .= "<input class='button' value='GO' type='submit' onClick='return confirm(\"Are you sure?\");'></form>";
		return $out;
	}
}

?>
